/*
 * File Name      : EciB1.cs
 *
 * Product        : CICS Transaction Gateway
 *
 * Description    : This sample shows the basic use of the CICS Transaction
 *                  Gateway ECI API. It queries the Gateway for a list of
 *                  available servers and then launches a transaction on the
 *                  server chosen.
 *
 * Pre-Requisites : This sample requires the CICS Program EC01 to be installed
 *                  on your CICS server for operation. The CICS server must be
 *                  set up to return the contents of the commarea as ASCII
 *                  text. If the code page of the application is different
 *                  from the code page of the server, data conversion must be
 *                  performed at the server by making use of CICS-supplied
 *                  resource conversion capabilities, such as the DFHCNV macro
 *                  definitions.
 *
 *                  This sample must be compiled to target the x86 processor
 *                  architecture, rather than the default of Any CPU. If the
 *                  sample is compiled for Any CPU, it may fail to execute on
 *                  64 bit platforms.
 *
 *                                 
 * Licensed Materials - Property of IBM
 * 
 * 5724-I81 5655-R25
 * 
 * (c) Copyright IBM Corp. 2008
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * Status: Version 7 Release 2
 *                               
 *
 *         The following code is sample code created by IBM Corporation. This
 *         sample code is not part of any standard IBM product and is provided
 *         to you solely for the purpose of assisting you in the development of
 *         your applications.  The code is provided 'AS IS', without warranty
 *         or condition of any kind.  IBM shall not be liable for any damages
 *         arising out of your use of the sample code, even if IBM has been
 *         advised of the possibility of such damages.
 */

using System;
using System.Text;
using System.Collections.Generic;

//Namespace IBM.CTG contains all the CICS Transaction Gateway types
using IBM.CTG;

namespace IBM.CTG.Samples
{
   public class EciB1
   {
      // 
      public const string copyright = "Licensed Materials - Property of IBM "+
      "5724-I81 5655-R25 " +
      "(c) Copyright IBM Corp. 2008  All Rights Reserved. " +
      "US Government Users Restricted Rights - Use, duplication or " +
      "disclosure restricted by GSA ADP Schedule Contract with " +
      "IBM Corp.";
      // 


      //Name of the CICS program to execute
      private const string ProgramName = "EC01";

      //The size of the COMMAREA to send to the program
      private const int    CommareaLength = 18;

      //Maximum number of flow attempts
      private const int    MaxFlowAttempts = 3;
      

      //Objects used by the class
      private string gwyHostName = "localhost";
      private int    gwyPort = 2006;

      /// <summary>
      /// Application entry point.
      /// </summary>
      /// <param name="args">Command line arguments passed to the application.</param>
      private static void Main(string[] args)
      {
         EciB1 eciApp = new EciB1();

         //Display program banner
         Console.WriteLine("CICS Transaction Gateway Basic ECI Sample 1");
         Console.WriteLine();

         //Display usage information
         Console.WriteLine("Usage: EciB1.exe [Gateway host name] [Gateway port number]");
         Console.WriteLine();

         if (eciApp.ParseArguments(args)) {
            try {
               eciApp.Start();

            } catch (Exception e) {
               //Output exception details
               Console.WriteLine(e.Message);
               Console.WriteLine(e.StackTrace);
            }
         }
      }

      /// <summary>
      /// Parses the command line arguments specified when the application was launched.
      /// </summary>
      /// <param name="args">An array of command line arguments.</param>
      /// <returns><see langword="true"/> if the arguments are parsed successfully; otherwise <see langword="false"/>.</returns>
      public bool ParseArguments(string[] args) {

         //Check argument count
         if (args.Length > 2) {
            for (int i = 2; i < args.Length; i++) {
               Console.WriteLine("ERROR: Unexpected argument: {0}", args[i]);
            }
            return false;
         }

         //Parse Gateway port number
         if (args.Length > 1) {
            if (!int.TryParse(args[1], out gwyPort)) {
               Console.WriteLine("ERROR: Invalid port number specified");
               return false;
            }
         }

         //Parse Gateway host name
         if (args.Length > 0) {
            gwyHostName = args[0];
         }

         Console.WriteLine("The address of the Gateway has been set to {0} port {1}", gwyHostName, gwyPort);
         Console.WriteLine();

         return true;
      }

      /// <summary>
      /// Connects to the CICS Transaction Gateway and flows an ECI request.
      /// </summary>
      /// <remarks>
      /// The user is prompted to select a CICS server to execute the EC01 program on.
      /// The contents of the COMMAREA returned from the request is displayed as ASCII text.
      /// </remarks>
      public void Start() {

         string cicsServer;
         EciRequest eciReq;

         //Open the Gateway connection with an infinite timeout period
         //The 'using' C# construct ensures the GatewayConnection is closed
         //when it goes out of scope, even if an exception is thrown
         
         //A System.Net.SocketException will be thrown if the connection
         //attempt fails, and an IBM.CTG.GatewayException will be thrown if
         //an error occurs in the Gateway daemon
         using (GatewayConnection gwyConnection = new GatewayConnection(gwyHostName, gwyPort)) {
         
            //Connected to the Gateway daemon

            //Get name of CICS server from the user
            cicsServer = SelectServer(gwyConnection);
            if (cicsServer == null) {
               //No servers defined, or user chose to quit
               return;
            }

            //Prepare the EciRequest
            eciReq = new EciRequest();
            eciReq.ServerName = cicsServer;
            eciReq.Program = ProgramName;
            eciReq.ExtendMode = EciExtendMode.EciNoExtend;
            eciReq.SetCommareaData(new byte[CommareaLength]);

            //Flow the request
            if (FlowRequest(gwyConnection, eciReq)) {

               //Display returned COMMAREA
               byte[] commarea = eciReq.GetCommareaData();

               //Display as hex
               Console.Write("  HEX: ");
               for (int i = 0; i < commarea.Length; i++) {
                  Console.Write("{0:x2}", commarea[i]);
               }
               Console.WriteLine();

               //Convert the byte data into a string, treating the data
               //as ASCII encoded characters. This relies on the CICS program
               //returning ASCII text in the COMMAREA
               string commareaStr = Encoding.ASCII.GetString(commarea);
               Console.WriteLine("ASCII: {0}", commareaStr);
            }
         }
      }


      /// <summary>
      /// Flows the ECI request, up to a maximum of MaxFlowAttempts attempts.
      /// </summary>
      /// <returns><see langword="true"/> if the flow succeeds; <see langword="false"/> if MaxFlowAttempts is reached.</returns>
      /// <remarks>
      /// If the flow returns a security error, the user is prompted to enter CICS logon credentials.
      /// </remarks>
      private bool FlowRequest(GatewayConnection gwyConnection, EciRequest eciReq) {

         int attempts = 0;

         while (attempts < MaxFlowAttempts) {
            
            //Flow throws a System.Net.SocketException if a connection
            //failure occurs, or an IBM.CTG.GatewayException if an error
            //occurs in the Gateway daemon
            gwyConnection.Flow(eciReq);

            //Test return code
            switch (eciReq.EciReturnCode) {
            case EciReturnCode.EciNoError:
               //Flow succeeded
               return true;

            case EciReturnCode.EciErrSecurityError:
               //Security error, prompt for CICS logon credentials
               Console.Write("Enter CICS user id : ");
               eciReq.UserId = Console.ReadLine();
               Console.Write("Enter CICS password: ");
               eciReq.Password = Console.ReadLine();
               break;

            case EciReturnCode.EciErrTransactionAbend:
               //Transaction abended
               Console.WriteLine("The transaction abended: {0}", eciReq.AbendCode);
               return false;

            case EciReturnCode.EciErrNoCics:
               //CICS server not found
               Console.WriteLine("CICS server not found");
               return false;

            default:
               //Other error return code
               Console.WriteLine("ECI return code: {0} ({1})", eciReq.EciReturnCode.ToString(), (int) eciReq.EciReturnCode);
               return false;

            }

            //Increment number of attempts
            attempts++;
         }

         //Number of attempts exceeded MaxFlowAttempts
         return false;
      }


      /// <summary>
      /// Prompts the user to select a server to execute the EC01 program on.
      /// </summary>
      /// <returns>The name of the CICS server, as defined in the CICS Transaction Gateway.</returns>
      private string SelectServer(GatewayConnection gwyConnection) {

         IList<CicsServer> servers;
         string            chosenServer = null;

         //Retrieve list of CICS servers from the Gateway
         
         //ListSystems throws a System.Net.SocketException if a connection
         //failure occurs, or an IBM.CTG.GatewayException if an error occurs
         //in the Gateway daemon
         servers = gwyConnection.ListSystems();

         if (servers.Count == 0) {
            //There are no servers defined in the Gateway
            Console.WriteLine("No CICS servers have been defined in the Gateway");
            return null;
         }

         while (chosenServer == null) {

            //Display a list of available CICS servers
            Console.WriteLine("CICS servers defined in the Gateway:");

            for (int i = 0; i < servers.Count; i++) {
               Console.WriteLine("   {0,2}. {1,-8} - {2}", (i + 1), servers[i].Name, servers[i].Description);
            }
            Console.WriteLine();

            //Prompt for user selection
            Console.Write("Choose server to connect to, or Q to quit: ");
            chosenServer = Console.ReadLine();

            //Parse user selection
            if (chosenServer.Equals("Q", StringComparison.InvariantCultureIgnoreCase)) {
               //User chose to quit
               return null;
            }

            int serverNum;
            if (int.TryParse(chosenServer, out serverNum)) {

               //Validate number
               if ((serverNum < 1) || (serverNum > servers.Count)) {
                  Console.WriteLine("Invalid selection");
                  chosenServer = null;
               } else {
                  chosenServer = servers[serverNum - 1].Name;
               }

            } else {
               //Input was not a number
               Console.WriteLine("Invalid selection");
               chosenServer = null;
            }

            Console.WriteLine();
         }

         return chosenServer;
      }

   }
}
